/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <libxml/tree.h>
#include "loader.h"
#include "diagram_graphic.h"
#include <map>
#include <stdio.h>

bool
FileLoader::goodXML(void *doc, OrderedSet *os, string filename)
{
  if (os) {
    os->fileSaved();
    os->setFilename(filename);
  }
  xmlFreeDoc((xmlDocPtr)doc);
  return true;
}

bool
FileLoader::failXML(void *doc, OrderedSet *os)
{
  if (os) orderedsets->unrefSet(os);
  xmlFreeDoc((xmlDocPtr)doc);
  return false;
}

bool
FileLoader::loadFromXML(string filename, OrderedSet *&os)
{
  xmlDocPtr doc;
  xmlNodePtr root_node, elements_node, element_node, vertex_node, x_node, 
    y_node, orders_node, order_node, diagram_node, dummies_node, dummy_node,
    edges_node, edge_node;
  string elem1_name, elem2_name;
  char *attr;
  Coord_t vertex_x, vertex_y;
  Vertex *vertex, *vert;
  Element *elem1, *elem2;
  ElementId dummy_id1, dummy_id2;
  map<ElementId, Vertex> dummy_map;
  map<ElementId, Vertex>::iterator iter_m1, iter_m2;

  doc = xmlParseFile(filename.c_str());
  if (!doc) return false;
  root_node = xmlDocGetRootElement(doc);
  if ((!root_node) || (xmlStrcmp(root_node->name, (xmlChar *)"ordered_set")))
    return failXML(doc, NULL);
  attr = (char *)xmlGetProp(root_node, (xmlChar *)"data_type");
  if (attr == NULL) return failXML(doc, NULL);
  os = NULL;
  if (string(attr) == "tree")
    os = orderedsets->newSet(OrderedSet::DATA_TREE);
  else if (string(attr) == "list")
    os = orderedsets->newSet(OrderedSet::DATA_LIST);
  else if (string(attr) == "array")
    os = orderedsets->newSet(OrderedSet::DATA_ARRAY);
  if (!os) return failXML(doc, NULL);
  elements_node = root_node->children; 
  if (xmlIsBlankNode(elements_node)) elements_node = elements_node->next;
  if (!elements_node) return goodXML(doc, os, filename);
  if (xmlStrcmp(elements_node->name, (xmlChar *)"elements"))
    return failXML(doc, os);
  (elements_node->children) ? 
    element_node = elements_node->children : element_node = NULL;
  if (xmlIsBlankNode(element_node)) element_node = element_node->next;
  while (element_node) {
    if (xmlStrcmp(element_node->name, (xmlChar *)"element"))
      return failXML(doc, os);
    attr = (char *)xmlGetProp(element_node, (xmlChar *)"name");
    if (attr == NULL) return failXML(doc, os);
    elem1_name = attr;
    vertex = NULL;
    dummy_id1 = NoElement;
    vertex_node = element_node->children;
    if (xmlIsBlankNode(vertex_node)) vertex_node = vertex_node->next;
    if (vertex_node) {
      if (xmlStrcmp(vertex_node->name, (xmlChar *)"vertex")) 
	return failXML(doc, os);
      attr = (char *)xmlGetProp(vertex_node, (xmlChar *)"dummy_id");
      if (attr)	sscanf(attr, "%hi", &dummy_id1);
      x_node = vertex_node->children; 
      if (xmlIsBlankNode(x_node)) x_node = x_node->next;
      if ((!x_node) || (xmlStrcmp(x_node->name, (xmlChar *)"x")))
	return failXML(doc, os);
      attr = (char *)xmlNodeListGetString(doc, x_node->children, 1);
      vertex_x = 0;
      if (attr) sscanf(attr, "%hi", &vertex_x);
      y_node = x_node->next; 
      if (xmlIsBlankNode(y_node)) y_node = y_node->next;
      if ((!y_node) || (xmlStrcmp(y_node->name, (xmlChar *)"y")))
	return failXML(doc, os);
      attr = (char *)xmlNodeListGetString(doc, y_node->children, 1);
      vertex_y = 0;
      if (attr) sscanf(attr, "%hi", &vertex_y);
      vertex = new Vertex; vertex->dummy = dummy_id1;
      vertex->x = vertex_x; vertex->y = vertex_y;
    }
    elem1 = os->newElement(elem1_name, vertex);
    if (!elem1) return failXML(doc, os);
    if (vertex) delete vertex;
    if (dummy_id1) {
      vertex = os->getDiagram()->getVertex(*elem1);
      dummy_map[dummy_id1] = *vertex;
    }
    element_node = element_node->next;
    if (xmlIsBlankNode(element_node)) element_node = element_node->next;
  }
  orders_node = elements_node->next;
  if (xmlIsBlankNode(orders_node)) orders_node = orders_node->next;
  if (!orders_node) return goodXML(doc, os, filename);
  if (xmlStrcmp(orders_node->name, (xmlChar *)"orders")) 
    return failXML(doc, os);
  (orders_node->children) ?
    order_node = orders_node->children->next : order_node = NULL;
  if (xmlIsBlankNode(order_node)) order_node = order_node->next;
  while (order_node) {
    if (xmlStrcmp(order_node->name, (xmlChar *)"order"))
      return failXML(doc, os);
    element_node = order_node->children;
    if (xmlIsBlankNode(element_node)) element_node = element_node->next;
    if ((!element_node) || 
	(xmlStrcmp(element_node->name, (xmlChar *)"element"))) 
      return failXML(doc, os);
    attr = (char *)xmlGetProp(element_node, (xmlChar *)"name");
    if (attr == NULL) return failXML(doc, os);
    elem1_name = attr;
    element_node = element_node->next;
    if (xmlIsBlankNode(element_node)) element_node = element_node->next;
    if ((!element_node) || 
	(xmlStrcmp(element_node->name, (xmlChar *)"element")))
      return failXML(doc, os);
    attr = (char *)xmlGetProp(element_node, (xmlChar *)"name");
    if (attr == NULL) return failXML(doc, os);
    elem2_name = attr;
    if ((elem1_name != "") && (elem2_name != "")) {
      elem1 = os->getElement(elem1_name);
      elem2 = os->getElement(elem2_name);
      if ((!elem1) || (!elem2)) return failXML(doc, os);
      os->orderElements(*elem1, *elem2);
    }
    order_node = order_node->next;
    if (xmlIsBlankNode(order_node)) order_node = order_node->next;
  }
  diagram_node = orders_node->next; 
  if (xmlIsBlankNode(diagram_node)) diagram_node = diagram_node->next;
  if (!diagram_node) return goodXML(doc, os, filename);
  if (xmlStrcmp(diagram_node->name, (xmlChar *)"diagram"))
    return failXML(doc, os);
  dummies_node = diagram_node->children; 
  if (xmlIsBlankNode(dummies_node)) dummies_node = dummies_node->next;
  if (!dummies_node) return goodXML(doc, os, filename);
  if (xmlStrcmp(dummies_node->name, (xmlChar *)"dummies"))
    return failXML(doc, os);
  (dummies_node->children) ? 
    dummy_node = dummies_node->children : dummy_node = NULL;
    if (xmlIsBlankNode(dummy_node)) dummy_node = dummy_node->next;
  while (dummy_node) {
    if (xmlStrcmp(dummy_node->name, (xmlChar *)"dummy"))
      return failXML(doc, os);
    vertex = NULL;
    attr = (char *)xmlGetProp(dummy_node, (xmlChar *)"id");
    if (attr == NULL) return failXML(doc, os);
    sscanf(attr, "%hi", &dummy_id1);
    x_node = dummy_node->children; 
    if (xmlIsBlankNode(x_node)) x_node = x_node->next;
    if ((!x_node) || (xmlStrcmp(x_node->name, (xmlChar *)"x")))
      return failXML(doc, os);
    attr = (char *)xmlNodeListGetString(doc, x_node->children, 1);
    vertex_x = 0;
    if (attr) sscanf(attr, "%hi", &vertex_x);
    y_node = x_node->next; 
    if (xmlIsBlankNode(y_node)) y_node = y_node->next;
    if ((!y_node) || (xmlStrcmp(y_node->name, (xmlChar *)"y")))
      return failXML(doc, os);
    attr = (char *)xmlNodeListGetString(doc, y_node->children, 1);
    vertex_y = 0;
    if (attr) sscanf(attr, "%hi", &vertex_y);
    vertex = new Vertex; 
    vertex->element = NoElement; vertex->dummy = dummy_id1;
    vertex->x = vertex_x; vertex->y = vertex_y;
    vert = os->getDiagram()->addVertex(*vertex);
    if (!vert) return failXML(doc, os);
    if (vertex) delete vertex;
    dummy_map[dummy_id1] = *vert;
    dummy_node = dummy_node->next;
    if (xmlIsBlankNode(dummy_node)) dummy_node = dummy_node->next;
  }
  edges_node = dummies_node->next;
  if (xmlIsBlankNode(edges_node)) edges_node = edges_node->next;
  if (!edges_node) return goodXML(doc, os, filename);
  if (xmlStrcmp(edges_node->name, (xmlChar *)"edges")) 
    return failXML(doc, os);
  (edges_node->children) ?
    edge_node = edges_node->children->next : edge_node = NULL;
  if (xmlIsBlankNode(edge_node)) edge_node = edge_node->next;
  while (edge_node) {
    if (xmlStrcmp(edge_node->name, (xmlChar *)"edge"))
      return failXML(doc, os);
    dummy_node = edge_node->children;
    if (xmlIsBlankNode(dummy_node)) dummy_node = dummy_node->next;
    if ((!dummy_node) || 
	(xmlStrcmp(dummy_node->name, (xmlChar *)"dummy"))) 
      return failXML(doc, os);
    attr = (char *)xmlGetProp(dummy_node, (xmlChar *)"id");
    if (attr == NULL) return failXML(doc, os);
    sscanf(attr, "%hi", &dummy_id1);
    dummy_node = dummy_node->next;
    if (xmlIsBlankNode(dummy_node)) dummy_node = dummy_node->next;
    if ((!dummy_node) || 
	(xmlStrcmp(dummy_node->name, (xmlChar *)"dummy")))
      return failXML(doc, os);
    attr = (char *)xmlGetProp(dummy_node, (xmlChar *)"id");
    if (attr == NULL) return failXML(doc, os);
    sscanf(attr, "%hi", &dummy_id2);
    if (dummy_id1 && dummy_id2) {
      iter_m1 = dummy_map.find(dummy_id1);
      iter_m2 = dummy_map.find(dummy_id2);
      if ((iter_m1 == dummy_map.end()) || (iter_m2 == dummy_map.end())) 
	return failXML(doc, os);
      os->getDiagram()->addDummyEdge((*iter_m1).second, (*iter_m2).second);
    }
    edge_node = edge_node->next;
    if (xmlIsBlankNode(edge_node)) edge_node = edge_node->next;
  }
  return goodXML(doc, os, filename);
}

FileLoader::FileLoader(OrderedSets *oss)
{
  orderedsets = oss;
}

FileLoader::error_t
FileLoader::load(string filename, type_t type, OrderedSet *&os)
{
  string::size_type pos;
  string stype;
  FILE *file = NULL;
  bool load_ok;

  if (type == NOTYPE) {
    pos = filename.rfind('.');
    if (pos != filename.npos) stype = filename.substr(pos);
    if (stype == ".lat") type = XML;
  }
  if (type == NOTYPE) return ETYPE;
  file = fopen(filename.c_str(), "r");
  if (!file) return EACCESS;
  else {
    if (getc(file) == EOF) { fclose(file); return EACCESS; }
    fclose(file);
  }
  switch (type) {
  case XML: load_ok = loadFromXML(filename, os); break;
  default: ;
  }
  if (!load_ok) return ELOAD;
  return EOK;
}
