/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "orderedsets.h"
#include "options.h"
#include "appwindow.h"
#include "diagramwindow.h"
#include "loader.h"
#include "saver.h"
#include "dialogs.h"
#include "diagram_graphic.h"
#include "i18n.h"
#include <locale.h>
#include <qapplication.h>
#include <qtextcodec.h>
#include <stdio.h>

int
main(int argc, char *argv[])
{
  QApplication *app;
  Options *options;
  OrderedSets *orderedsets;
  AppWindow *window;
  string filename;
  FileLoader *loader;
  FileSaver *saver;
  OpenFileDialog *openfile_dialog;
  OrderedSet *os;
  bool load_ok, run_gui = true;
  FileLoader::error_t load_err = FileLoader::EOK;
  FileSaver::error_t save_err = FileSaver::EOK;
  Diagram_Graphic::generationParams_t generation_params;
  RearrangeDiagramDialog *rearrangediagram_dialog;
  DiagramWindow *diagramwindow;

  setlocale(LC_ALL, "");
  bindtextdomain(PACKAGE, LOCALEDIR);
#ifdef WIN32
  //bind_textdomain_codeset(PACKAGE, "CP1250");
#endif
  textdomain(PACKAGE);
  // app  
  app = new QApplication(argc, argv);
  options = new Options(argc, argv);
  if (options->getHelp()) { options->printHelp(); return 0; }
  if (options->getVersion()) { options->printVersion(); return 0; }
  // TODO: will be an option in preferences
  if (options->getSave()) run_gui = false;
  orderedsets = new OrderedSets;
  if (run_gui) {
    window = new AppWindow(orderedsets);
    openfile_dialog = new OpenFileDialog(window, orderedsets);
  }
  else loader = new FileLoader(orderedsets);
  while ((filename = options->getNextFilename()) != "") {
    if (run_gui) load_ok = openfile_dialog->openFile(filename, os);
    else load_err = loader->load(filename, FileLoader::NOTYPE, os);
    if ((run_gui && load_ok) || 
	(!run_gui && (load_err == FileLoader::EOK))) {
      if (run_gui) 
	diagramwindow = new DiagramWindow(window, orderedsets, os);
      if (options->getDiagram()) {
	if ((options->getDiagramMethod() == 
	     Options::OPTION_DIAGRAM_METHOD_NONE) && run_gui) {
	  // TODO: will be an option in preferences
	  rearrangediagram_dialog = 
	    new RearrangeDiagramDialog(os, diagramwindow);
	  rearrangediagram_dialog->exec();
	}
	else {
	  generation_params = os->getDiagram()->getGenerationParams();
	  switch (options->getDiagramMethod()) {
	  case Options::OPTION_DIAGRAM_METHOD_LEVEL:
	    generation_params.generation_method =
	      Diagram_Graphic::DIAGRAM_LEVEL;
	    break;
	  case Options::OPTION_DIAGRAM_METHOD_GEOMETRIC:
	  case Options::OPTION_DIAGRAM_METHOD_NONE:
	    generation_params.generation_method = 
	      Diagram_Graphic::DIAGRAM_GEOMETRIC;
	    break;
	  case Options::OPTION_DIAGRAM_METHOD_LAYER:
	    generation_params.generation_method = 
	      Diagram_Graphic::DIAGRAM_LAYER;
	    break;
	  default:;
	  }
	  os->save_UndoRedo();
	  os->getDiagram()->generateDiagram(generation_params);
	}
      }
      if (options->getSave()) {
	saver = new FileSaver(os);
	save_err = saver->save(filename, FileSaver::NOTYPE);
	delete saver;
	if (save_err == FileSaver::ETYPE)
	  fprintf(stderr, "%s\n%s\n", _("Save failed!"),
		  (filename + ": " + _("Unknown file type.")).c_str());
	else if (save_err == FileSaver::EACCESS)
	  fprintf(stderr, "%s\n%s\n", _("Save failed!"), 
		  (filename + " " + _("is unwritable.")).c_str());
	else if (save_err == FileSaver::ESAVE)
	  fprintf(stderr, "%s\n%s\n", _("Save failed!"), 
		  (filename + ": " + _("Save error.")).c_str());
      }
      if (!run_gui) orderedsets->unrefSet(os);
    }
    else if (load_err == FileLoader::ETYPE)
      fprintf(stderr, "%s\n%s\n", _("Load failed!"), 
	      (filename + ": " + _("Unknown file type.")).c_str());
    else if (load_err == FileLoader::EACCESS)
      fprintf(stderr, "%s\n%s\n", _("Load failed!"), 
	      (filename + " " + _("is not present or is unreadable.")).
	      c_str());
    else if (load_err == FileLoader::ELOAD)
      fprintf(stderr, "%s\n%s\n", _("Load failed!"), 
	      (filename + ": " + _("Load error.")).c_str());
  }
  if (run_gui) delete openfile_dialog;
  else delete loader;
  // run app
  if (run_gui) { app->setMainWidget(window); app->exec(); }
  delete orderedsets;
  delete options;
  delete app;

  return 0;
}
